import os
import sys
import struct


class Size:

    def __init__(self):
        # Try each method, the first not to fail
        #   is saved to be used in later requests.
        self.method = None
        for method in range(5):
            if self.getSize(method):
                self.method = method
                break

    def getSize(self, method=None):
        if method is None:
            method = self.method

        if method == 0:
            # Try stdin, stdout, stderr
            for fd in (0, 1, 2):
                try:
                    import termios
                    import fcntl
                    s = list(struct.unpack("hh",
                                           fcntl.ioctl(
                                               fd, termios.TIOCGWINSZ, "1234")
                                           ))
                    s.reverse()
                    return tuple(s)
                except:
                    return False
        elif method == 1:
            # Try os.ctermid()
            try:
                import termios
                import fcntl
                fd = os.open(os.ctermid(), os.O_RDONLY)
                try:
                    s = list(struct.unpack("hh",
                                           fcntl.ioctl(
                                               fd, termios.TIOCGWINSZ, "1234")
                                           ))
                    s.reverse()
                    return tuple(s)
                finally:
                    os.close(fd)
            except:
                return False
        elif method == 2:
            # Try `stty size`
            try:
                size = os.popen("stty size", "r").read().split()
                s = list(int(x) for x in size)
                s.reverse()
                return tuple(s)
            except:
                return False
        elif method == 3:
            # Try environment variables
            try:
                return tuple(
                    int(os.getenv(var)) for var in ("COLUMNS", "LINES")
                )
            except:
                return False
        elif method == 4:
            # I give up. Return default
            return (80, 25)
        return None

    def __repr__(self):
        return 'Size{!r}'.format(self.getSize())


def supportedChars(*tests):
    """
        Takes any number of strings, and returns the first one
            the terminal encoding supports. If none are supported
            it returns '?' the length of the first string.
    """
    for test in tests:
        try:
            test.encode(sys.stdout.encoding)
            return test
        except UnicodeEncodeError:
            pass
    return '?' * len(tests[0])


def main():
    size = Size()
    oldS = None
    while True:
        s = size.getSize()
        if not s == oldS:
            print(s)
        oldS = s

if __name__ == '__main__':
    main()



import time
import random

import graphics as g


class Car(g.shapes.Image):

    def __init__(self, length=5):
        super(Car, self).__init__()
        self.length = length

    def genImage(self):
        l = self.length
        return [[0] + [1 for i in range(2, l)] + [0],
                [1 for i in range(l)],
                [0, 1] + [0 for i in range(4, l)] + [1, 0]]


def genGround(width, height, llimit, ulimit):
    image = [[] for i in range(height)]

    groundPos = int((ulimit + llimit) / 3) * 2

    for i in range(width):

        change = ulimit + 1  # Be sure the 1st iteration of the while will run.
        while change + groundPos <= llimit or change + groundPos >= ulimit:
            change = random.randint(-2, 2)

        yDiff = []
        if change > 0:
            yDiff = list(range(groundPos + 1, groundPos + change))
        elif change < 0:
            yDiff = list(range(groundPos + change, groundPos))

        groundPos += change

        for y, row in enumerate(image):
            if y == groundPos:
                image[y].append(chr(0x25EF))
            elif y in yDiff:
                if change < 0:
                    try:
                        image[y][-1] = chr(0x25FF)
                    except IndexError:
                        pass
                    image[y].append(' ')
                else:
                    image[y].append(chr(0x25FA))
            else:
                image[y].append(False)

    return image


def main():

    FPS = 15
    screen = g.Canvas(fullscreen=True, size=(36, 20), wrap=True)

    frameCount = g.Sprite(g.shapes.Text(''), color=g.colors.RED)
    car = g.Sprite(
        Car(),
        color=g.colors.WHITE
    )

    llimit = screen.height
    ulimit = car.image.height
    grndTerrain = genGround(screen.width,
                            screen.height,
                            car.image.height,
                            screen.height)

    ground = g.Sprite(
        g.shapes.CustImage(grndTerrain),
        color=g.colors.GREEN
    )

    screen.sprites.append(frameCount)
    screen.sprites.append(ground)
    screen.sprites.append(car)

    frame = 0
    frame1 = 0
    t = time.time()
    with g.NonBlockingInput() as nbi:
        while True:
            frame += 1
            frameCount.image.text = str(frame) + ':' + str(frame1)

            ch = nbi.char()
            if ch == '.':
                car.move(g.RIGHT)
            if ch == ',':
                car.move(g.LEFT)
            if ch == '/':
                car.image.length += 1
            if ch == '\\':
                car.image.length -= 1

            car.move(g.UP)
            while not car.touching(screen, side=0):
                car.move(g.DOWN)
            while car.touching(screen, side=0):
                car.move(g.UP)
            car.move(g.DOWN)

            if time.time() >= t + (1 / FPS):
                frame1 += 1
                t = time.time()
                print(screen)

if __name__ == '__main__':
    main()



#Copyright 2008-2009  Marius Muja (mariusm@cs.ubc.ca). All rights reserved.
#Copyright 2008-2009  David G. Lowe (lowe@cs.ubc.ca). All rights reserved.
#
#THE BSD LICENSE
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#1. Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
#THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
#IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
#OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
#THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.



from rp.libs.pyflann.exceptions import FLANNException
import numpy
have_h5py = True
try:
    import h5py
except Exception as e:
    have_h5py = False

if not have_h5py:
    def __missing_h5py(*args,**kwargs):
        raise FLANNException("h5py library not found")
    check = __missing_h5py
    save = __missing_h5py
    load = __missing_h5py
    load_range = __missing_h5py
else:
    def check(filename):
        f = open(filename,"r")
        header = f.read(4)
        if header[1:4]=="HDF": return True
        return False
            

    def save(dataset, filename, **kwargs):
        if not isinstance(dataset,numpy.ndarray):
            raise FLANNException("Dataset must be in numpy format")
        try:
            if 'title' in kwargs:
                title_name = kwargs['title']
            else:
                title_name = "Dataset saved by pyflann"
            if 'dataset_name' in kwargs:
                dataset_name = kwargs['dataset_name']
            else:
                dataset_name = 'dataset'
            h5file = h5py.File(filename)
            h5file.create_dataset(dataset_name, data=dataset)
            h5file.close()
        except Exception as e:
            h5file.close()
            raise FLANNException(e)


    def load(filename, rows = -1, cols = -1, dtype = numpy.float32, **kwargs):
        try:
            h5file = h5py.File(filename)
            if 'dataset_name' in kwargs:
                dataset_name = kwargs['dataset_name']
            else:
                dataset_name = 'dataset'
            
            for node in list(h5file.keys()):
                if node == dataset_name:
                    data = numpy.array(h5file[node])
            h5file.close()
            return data
        except Exception as e:
            h5file.close()
            raise FLANNException(e)
            

    def load_range(filename, array_name, range):
        h5file = h5py.File(filename)
        dataset = h5file[array_name]
        return dataset[range[0]:range[1]]
